/*
 Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
 

  The MySQL Connector/J is licensed under the terms of the GPLv2
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most MySQL Connectors.
  There are special exceptions to the terms and conditions of the GPLv2 as it is applied to
  this software, see the FLOSS License Exception
  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.

  This program is free software; you can redistribute it and/or modify it under the terms
  of the GNU General Public License as published by the Free Software Foundation; version 2
  of the License.

  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with this
  program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth
  Floor, Boston, MA 02110-1301  USA

 */

package com.mysql.jdbc;

import java.sql.SQLException;
import java.util.Iterator;

import com.mysql.jdbc.DatabaseMetaData.IteratorWithCleanup;

public abstract class IterateBlock<T> {
	IteratorWithCleanup<T> iteratorWithCleanup;
	Iterator<T> javaIterator;
	boolean stopIterating = false;
	
	IterateBlock(IteratorWithCleanup<T> i) {
		this.iteratorWithCleanup = i;
		this.javaIterator = null;
	}
	
	IterateBlock(Iterator<T> i) {
		this.javaIterator = i;
		this.iteratorWithCleanup = null;
	}

	public void doForAll() throws SQLException {
		if (this.iteratorWithCleanup != null) {
			try {
				while (this.iteratorWithCleanup.hasNext()) {
					forEach(this.iteratorWithCleanup.next());
					
					if (this.stopIterating) {
						break;
					}
				}
			} finally {
				this.iteratorWithCleanup.close();
			}
		} else {
			while (this.javaIterator.hasNext()) {
				forEach(this.javaIterator.next());
				
				if (this.stopIterating) {
					break;
				}
			}
		}
	}

	abstract void forEach(T each) throws SQLException;
	
	public final boolean fullIteration() {
		return !this.stopIterating;
	}
}